from mag_tools.enums.base_enum import BaseEnum

class MemoryType(BaseEnum):
    DDR = (20, "DDR Memory")
    DDR2 = (21, "DDR2 Memory")
    DDR3 = (24, "DDR3 Memory")
    DDR4 = (26, "DDR4 Memory")
    DDR5 = (31, "DDR5 Memory")
    LPDDR = (27, "Low Power DDR Memory")
    LPDDR2 = (28, "Low Power DDR2 Memory")
    LPDDR3 = (29, "Low Power DDR3 Memory")
    LPDDR4 = (30, "Low Power DDR4 Memory")
    LPDDR5 = (32, "Low Power DDR5 Memory")
    UNKNOWN = (0, "Unknown Memory")


from mag_tools.enums.base_enum import BaseEnum


class Symbol(BaseEnum):
    """
    标点符号的枚举

    作者: xlcao
    版本: 1.3
    版权所有: Copyright (c) 2017 by Xiaolong Cao. All rights reserved.
    日期: 2017/7/13
    """
    SEMI_COLON = (";", "分号")
    COMMA = (",", "逗号")
    COLON = (":", "冒号")
    DOT = (".", "句点")
    BLANK = (" ", "空格")
    SINGLE_QUOTATION = ("'", "单引号")
    DOUBLE_QUOTATION = ("\"", "双引号")
    FORWARD_SLASH = ("/", "正斜杠")
    BACK_SLASH = ("\\", "反斜杠")
    UNDER_LINE = ("_", "下划线")
    DOUBLE_UNDER = ("__", "双下划线")
    HYPHEN = ("-", "连字符")
    EQUAL = ("=", "等号")
    QUESTION_MARK = ("?", "问号")
    EXCLAMATION_MARK = ("!", "感叹号")
    AMPERSAND = ("&", "与号")
    ASTERISK = ("*", "星号")
    OPEN_PAREN = ("(", "左圆括号")
    CLOSE_PAREN = (")", "右圆括号")
    OPEN_BRACE = ("{", "左花括号")
    CLOSE_BRACE = ("}", "右花括号")
    OPEN_BRACKET = ("[", "左方括号")
    CLOSE_BRACKET = ("]", "右方括号")
    ACCENT = ("`", "抑音符/反引号")
    TILDE = ("~", "波浪号/颚化号")
    POUND = ("#", "井号")
    DOLLAR = ("$", "美元符")
    AT = ("@", "AT符")
    VERTICAL_BAR = ("|", "分隔符")
    PLUS = ("+", "加号")
    CARET = ("^", "指数运算符")
    PERCENT = ("%", "百分号")
    CHINES_DOT = ("。", "中文句号")
    CHINES_COMMA = ("，", "中文逗号")
    CHINES_COLON = ("：", "中文冒号")
    CHINESE_SINGLE_QUOTATION = ("‘", "中文单引号")
    CHINESE_DOUBLE_QUOTATION = ("“", "中文双引号")
    CHINESE_OPEN_PAREN = ("（", "中文左圆括号")
    CHINESE_CLOSE_PAREN = ("）", "中文右圆括号")
    CHINESE_OPEN_BRACKET = ("【", "中文左方括号")
    CHINESE_CLOSE_BRACKET = ("】", "中文右方括号")
    CHINESE_CAESURA = ("、", "中文顿号")
    CHINESE_QUESTION_MARK = ("？", "中文问号")
    CHINESE_EXCLAMATION_MARK = ("！", "中文感叹号")
    CHINESE_SEMI_COLON = ("；", "中文分号")
    CHINESE_DOLLAR = ("￥", "人民币符")
    PLACE_HOLDER = ("%s", "SQL占位符")

    NONE = ("", "空字符")

    @staticmethod
    def is_symbol(c: str) -> bool:
        return Symbol.of_code(c) is not None



import unittest
from typing import Optional

from utils.common.class_utils import ClassUtils


class TestClassUtils(unittest.TestCase):
    def test_get_origin_type(self):
        print(ClassUtils.get_origin_type(Optional[str]))
        print(ClassUtils.get_origin_type(Optional[int]))


import random
from datetime import date, datetime
from typing import List, Optional

from mag_tools.enums.justify_type import JustifyType
from mag_tools.utils.data.list_utils import ListUtils
from mag_tools.utils.data.random_generator import RandomGenerator
from mag_tools.utils.data.string_utils import StringUtils


class StringFormat:
    @staticmethod
    def pad_string(string: str, padding_length: Optional[int] = None, justify: JustifyType = JustifyType.RIGHT, pad_char: Optional[str] = None) -> str:
        """
        将字符串用空格补充到指定长度，空格添加在字符串前。
        参数：
        :param justify: 对齐方式，空格补齐
        :param string: 原始数值或字符串
        :param padding_length: 目标长度
        :param pad_char: 分隔符号，为None则缺省为空格
        :return: 补充空格后的字符串
        """
        if string is None:
            raise ValueError('字符串不是能为空')

        pad_char = pad_char if pad_char else ' '
        pad_len = max(len(string), padding_length) if padding_length is not None else len(string)

        padding_size = pad_len - len(string) #要填充的占位符个数
        if justify == JustifyType.RIGHT:
            return f"{pad_char * padding_size}{string}"
        elif justify == JustifyType.CENTER:
            left_padding = pad_char * (padding_size // 2)
            right_padding = pad_char * (padding_size - padding_size // 2)
            return  f'{left_padding}{string}{right_padding}'
        else:
            return f"{string}{pad_char * padding_size}"

    @staticmethod
    def pad_text(strings: List[str],
                 padding_length: Optional[int] = None,
                 justify: JustifyType = JustifyType.RIGHT,
                 number_per_line: Optional[int] = None,
                 at_header: Optional[str] = None,
                 at_end: Optional[str] = None,
                 pad_char: Optional[str] = None) -> List[str]:
        """
        将字符串数组按指定格式拼接为文本
        :param strings: 字符串数组
        :param padding_length: 填写后长度，为None则为原字段长度
        :param justify: 对齐方式
        :param number_per_line: 每行字段个数，为None表示不限定
        :param at_header: 文本首的填充内容
        :param at_end: 文本尾的填充内容
        :param pad_char: 分隔符号
        """
        if number_per_line is None:
            number_per_line = len(strings)
        at_header = at_header if at_header else ''
        at_end = at_end if at_end else ''
        pad_char = pad_char if pad_char else ' '

        max_length = max(len(str(s)) for s in strings)
        padding_length = max(padding_length, max_length) if padding_length is not None else None

        items = [StringFormat.pad_string(s, padding_length, justify) for s in strings]
        blocks = ListUtils.split(items, number_per_line)
        lines = [f'{at_header}{pad_char.join(block)}{at_end}' for block in blocks]
        return lines

    @staticmethod
    def format_function(template: Optional[str]) -> Optional[str]:
        """
        格式化字符串，处理{}中的特定函数，如果不是函数，则直接返回
        """
        if template is None or not all(c in template for c in '()'):
            return template

        while '{' in template and '}' in template:
            start = template.index('{')
            end = template.index('}', start)
            expression = template[start + 1:end]

            # 解析函数名和参数
            func_name, args = StringUtils.parse_function(expression)

            # 调用相应的函数
            if func_name == 'random_text':
                result = RandomGenerator.ascii_letters(int(args[0]))
            elif func_name == 'random_string':
                result = RandomGenerator.string(int(args[0]))
            elif func_name == 'random_chinese':
                length = int(args[0]) if args[0] else random.randint(1, 100)
                result = RandomGenerator.chinese_string(length)
            elif func_name == 'random_number':
                result = RandomGenerator.number_string(int(args[0]))
            elif func_name == 'random_int':
                min_value = int(args[0] if len(args) >= 2 else 1)
                max_value = int(args[1] if len(args) >= 2 else args[0])

                result = str(random.randint(min_value, max_value))
            elif func_name == 'random_date':
                year = int(args[0]) if len(args) > 0 else date.today().year
                fmt = args[1] if len(args) > 1 else "%Y%m%d"
                result = RandomGenerator.date(year).strftime(fmt)
            elif func_name == 'today':
                fmt = args[0] if len(args) > 0 else "%Y%m%d"
                result = date.today().strftime(fmt)
            elif func_name == 'current':
                fmt = args[0] if len(args) > 0 else "%Y-%m-%d %H:%M:%S"
                result = datetime.now().strftime(fmt)
            else:
                raise ValueError(f"未知的函数: {func_name}")

            # 替换模板中的表达式
            template = template[:start] + result + template[end + 1:]

        return template




from typing import List, Optional


class BytesUtils:
    """
    字节数组的工具类
    """
    @staticmethod
    def bytes_to_int(ba: bytes) -> int:
        """
        将byte[]转换为int型
        :param ba: 源byte[]
        :return: int型数值
        """
        value = 0
        for b in ba:
            value = (value << 8) | (b & 0xff)
        return value

    @staticmethod
    def bytes_to_string(ba: bytes, encoding: str = 'UTF-8') -> str:
        """
        把一个字节流转换成指定字符集的字符串
        :param ba: byte[] 字节流
        :param encoding: 字符集
        :return: 返回字符串
        """
        try:
            return ba.decode(encoding)
        except UnicodeDecodeError:
            return ba.decode()

    @staticmethod
    def bytes_to_array(ba: bytes) -> Optional[bytes]:
        """
        将byte数组转换为Byte[]
        :param ba: byte数组
        :return: Byte[]
        """
        return None if ba is None else ba

    @staticmethod
    def get_int_from_bytes(ba: bytes, idx: int) -> int:
        if idx + 4 > len(ba):
            return -1
        ba_int = ba[idx:idx + 4]
        return BytesUtils.bytes_to_int(ba_int)

    @staticmethod
    def get_string_from_bytes(ba: bytes, idx: int, length: int) -> str:
        ba_ret = ba[idx:idx + length]
        return BytesUtils.bytes_to_string(ba_ret)

    @staticmethod
    def get_bytes_from_bytes(ba: bytes, idx: int, length: int) -> bytes:
        ba_ret = ba[idx:idx + length]
        return ba_ret

    @staticmethod
    def int_to_bytes(value: int) -> bytes:
        b = bytearray(4)
        for i in range(4):
            b[3 - i] = value & 0xff
            value >>= 8
        return bytes(b)

    @staticmethod
    def string_to_bytes(value: str, encoding: str = 'UTF-8') -> bytes:
        try:
            return value.encode(encoding)
        except UnicodeEncodeError:
            return value.encode()

    @staticmethod
    def length(value: str, encoding: str = 'UTF-8') -> int:
        ba_stream = BytesUtils.string_to_bytes(value, encoding)
        return len(ba_stream) if ba_stream else -1

    @staticmethod
    def set_int_in_bytes(value: int, ba: bytearray, idx: int):
        if idx + 4 > len(ba):
            return
        ba_int = BytesUtils.int_to_bytes(value)
        ba[idx:idx + 4] = ba_int

    @staticmethod
    def set_string_in_bytes(value: str, ba: bytearray, idx: int, encoding: str = 'UTF-8'):
        ba_value = BytesUtils.string_to_bytes(value, encoding)
        if not value or idx + len(ba_value) > len(ba):
            return
        ba[idx:idx + len(ba_value)] = ba_value

    @staticmethod
    def set_bytes_in_bytes(value: bytes, ba: bytearray, idx: int):
        if not value or idx + len(value) > len(ba):
            return
        ba[idx:idx + len(value)] = value

    @staticmethod
    def link_byte_array(buf1: bytes, buf2: bytes) -> bytes:
        return buf1 + buf2

    @staticmethod
    def get_bytes(array: bytes, start: int, length: int) -> bytes:
        if start + length > len(array):
            length = len(array) - start
        return array[start:start + length]

    @staticmethod
    def bytes_to_hex(data: bytes) -> str:
        return ''.join(f'{b:02X}' for b in data)

    @staticmethod
    def hex_to_bytes(hex_str: str) -> Optional[bytes]:
        if not hex_str or len(hex_str) % 2 != 0:
            return None
        return bytes.fromhex(hex_str)

    @staticmethod
    def merge_bytes(vec_data: List[bytes]) -> Optional[bytes]:
        if not vec_data:
            return None
        return b''.join(vec_data)

    @staticmethod
    def split_stream(data: bytes, block_size: int) -> List[bytes]:
        vec_data = [data[i:i + block_size] for i in range(0, len(data), block_size)]
        return vec_data

    @staticmethod
    def trim_bytes(byte_array: bytes) -> Optional[bytes]:
        if byte_array is None:
            return None
        return byte_array.strip(b' ')

    @staticmethod
    def char_to_byte(c: str) -> int:
        return "0123456789ABCDEF".index(c)



import json
from dataclasses import asdict, is_dataclass
from enum import Enum
from typing import Any, Dict, Type, TypeVar, Optional
from datetime import datetime, date
from decimal import Decimal

from mag_tools.jsonparser.json_encoder import JsonEncoder
from mag_tools.enums.base_enum import BaseEnum
from mag_tools.utils.data.map_utils import MapUtils

K = TypeVar('K')
V = TypeVar('V')


class EasyMap(Dict[K, V]):
    """
    Map简化操作类

    @author xlcao
    @version 2.2
    @copyright Copyright (c) 2015
    """

    def __init__(self, map_: Optional[Dict[K, V]] = None) -> None:
        super().__init__(map_ if map_ else {})

    def add(self, key: K, value: V):
        """
        添加键值及对应的数值

        :param key: 键值
        :param value: 数值
        :return: self
        """
        if value is not None:
            self[key] = value
        return self

    def get_string(self, key: Any) -> Optional[str]:
        """
        从Map中取字符串

        :param key: 键值
        :return: 字符串
        """
        return str(self.get(key)) if self.get(key) is not None else None

    def get_byte(self, key: Any) -> Optional[bytes]:
        """
        从Map中取字节

        :param key: 键值
        :return: 字节
        """
        value = self.get_string(key)
        return bytes(value, 'utf-8') if value else None

    def get_int(self, key: Any) -> Optional[int]:
        """
        从Map中取整数

        :param key: 键值
        :return: 整数
        """
        value = self.get_string(key)
        try:
            return int(value) if value else None
        except ValueError:
            return None

    def get_float(self, key: Any) -> Optional[float]:
        """
        从Map中取Double

        :param key: 键值
        :return: Double
        """
        value = self.get_string(key)
        try:
            return float(value) if value else None
        except ValueError:
            return None

    def get_decimal(self, key: Any) -> Optional[Decimal]:
        """
        从Map中取BigDecimal

        :param key: 键值
        :return: BigDecimal
        """
        value = self.get_string(key)
        try:
            return Decimal(value) if value else None
        except ValueError:
            return None

    def get_date(self, key: Any) -> Optional[date]:
        """
        从Map中取日期

        :param key: 键值
        :return: 日期
        """
        value = self.get_string(key)
        try:
            return datetime.strptime(value, '%Y-%m-%d').date() if value else None
        except ValueError:
            return None

    def get_datetime(self, key: Any) -> Optional[datetime]:
        """
        从Map中取日期时间

        :param key: 键值
        :return: 日期时间
        """
        value = self.get_string(key)
        try:
            return datetime.strptime(value, '%Y-%m-%d %H:%M:%S') if value else None
        except ValueError:
            return None

    def get_time(self, key: Any) -> Optional[datetime.time]:
        """
        从Map中取时间

        :param key: 键值
        :return: 时间
        """
        value = self.get_string(key)
        try:
            return datetime.strptime(value, '%H:%M:%S').time() if value else None
        except ValueError:
            return None

    def get_bool(self, key: Any) -> bool:
        """
        从Map中取布尔值

        :param key: 键值
        :return: 布尔值
        """
        value = self.get_string(key)
        return value.lower() in ('y', 'yes', 't', 'true', '1') if value else False

    def get_enum(self, key: Any, enum_type: Type[Enum]) -> Optional[V]:
        """
        从Map中取枚举

        :param key: 键值
        :param enum_type: 枚举类型
        :return: 枚举值
        """
        try:
            code = self.get_string(key)
            if code:
                if issubclass(enum_type, BaseEnum):
                    return enum_type.of_code(code) or enum_type[code]
                else:
                    return enum_type[code]
        except KeyError:
            return None

    def get_string_by_json(self, key: Any) -> Optional[str]:
        """
        从Map中取字符串（JSON）

        :param key: 键值
        :return: 字符串
        """
        return json.dumps(self.get(key))

    def get_int_by_json(self, key: Any) -> Optional[int]:
        """
        从Map中取整数（JSON）

        :param key: 键值
        :return: 整数
        """
        value = self.get_string_by_json(key)
        try:
            return int(value) if value else None
        except ValueError:
            return None

    def get_float_by_json(self, key: Any) -> Optional[float]:
        """
        从Map中取float（JSON）

        :param key: 键值
        :return: float
        """
        value = self.get_string_by_json(key)
        try:
            return float(value) if value else None
        except ValueError:
            return None

    def get_decimal_by_json(self, key: Any) -> Optional[Decimal]:
        """
        从Map中取Decimal（JSON）

        :param key: 键值
        :return: Decimal
        """
        value = self.get_string_by_json(key)
        try:
            return Decimal(value) if value else None
        except ValueError:
            return None

    def get_date_by_json(self, key: Any) -> Optional[date]:
        """
        从Map中取日期（JSON）

        :param key: 键值
        :return: 日期
        """
        value = self.get_string_by_json(key)
        try:
            return datetime.strptime(value, '%Y-%m-%d').date() if value else None
        except ValueError:
            return None

    def get_datetime_by_json(self, key: Any) -> Optional[datetime]:
        """
        从Map中取日期时间（JSON）

        :param key: 键值
        :return: 日期时间
        """
        value = self.get_string_by_json(key)
        try:
            return datetime.strptime(value, '%Y-%m-%d %H:%M:%S') if value else None
        except ValueError:
            return None

    def get_time_by_json(self, key: Any) -> Optional[datetime.time]:
        """
        从Map中取时间（JSON）

        :param key: 键值
        :return: 时间
        """
        value = self.get_string_by_json(key)
        try:
            return datetime.strptime(value, '%H:%M:%S').time() if value else None
        except ValueError:
            return None

    def get_bool_by_json(self, key: Any) -> bool:
        """
        从Map中取布尔值（JSON）

        :param key: 键值
        :return: 布尔值
        """
        value = self.get_string_by_json(key)
        return value.lower() in ('y', 'yes', 't', 'true', '1') if value else False

    def get_enum_by_json(self, key: Any, enum_type: Type[V]) -> Optional[V]:
        """
        从Map中取枚举（JSON）

        :param key: 键值
        :param enum_type: 枚举类型
        :return: 枚举值
        """
        try:
            code = self.get_string_by_json(key)
            if code:
                if issubclass(enum_type, BaseEnum):
                    return enum_type.of_code(code) or enum_type[code]
                else:
                    return enum_type[code]
        except KeyError:
            return None

    def put(self, key: K, value: V) -> V:
        """
        添加键值及对应的数值

        :param key: 键值
        :param value: 数值
        :return: 数值
        """
        return super().__setitem__(key, value)

    def to_map(self) -> Dict[K, V]:
        """
        转为Map

        :return: Map
        """
        return dict(self)

    def keys_to_hump(self) -> Dict[str, Any]:
        """
        将键值转为驼峰格式，包括为对象的成员变量中的变量
        """
        obj = {k: asdict(v) if is_dataclass(v) else v for k, v in self.items()}
        json_str = json.dumps(obj, cls=JsonEncoder) if obj else "{}"

        new_map = json.loads(json_str)
        return MapUtils.keys_to_hump(new_map)

    def get_by_json(self, key: Any, clazz: Type[V]) -> Optional[V]:
        """
        从Map中取对象（JSON）

        :param key: 键值
        :param clazz: 类类型
        :return: 对象
        """
        value = self.get(key)
        return json.loads(value, object_hook=clazz) if value else None
